from .movie_review import MovieReview
from .customer_review import CustomerReview
from .imdb import IMDB
from .multi_nli import MultiNLI
from .news_group20 import NewsGroup20
from .reuters_news import ReutersNews
from .ptb import PTB
from .wikitext2 import WikiText2
from .wikitext103 import WikiText103
from .text8 import Text8



import os
import tarfile
import shutil
from collections import namedtuple
import requests
from chazutsu.datasets.framework.xtqdm import xtqdm
from chazutsu.datasets.framework.dataset import Dataset
from chazutsu.datasets.framework.resource import Resource


class ReutersNews(Dataset):

    def __init__(self, kind="topics"):
        super().__init__(
            name="ReutersNews",
            site_url="http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/lyrl2004_rcv1v2_README.htm",
            download_url="https://s3-ap-northeast-1.amazonaws.com/dev.tech-sketch.jp/chakki/chazutsu/RCV1-v2.zip",
            description="Reuters news corpus that is annotated by 3 kinds of label (topics, industries, region)."
            )
        
        label_urls = {
            "topics": "http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/a08-topic-qrels/rcv1-v2.topics.qrels.gz",
            "industries": "http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/a09-industry-qrels/rcv1-v2.industries.qrels.gz",
            "regions": "http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/a10-region-qrels/rcv1-v2.regions.qrels.gz",
        }

        if kind not in label_urls:
            raise Exception("You have to choose kind from {}".format(",".join(label_urls.keys())))

        self.kind = kind
        self.label_url = label_urls[self.kind]
        self.label_file = "rcv1-v2.{}.qrels".format(self.kind)
        self.label_desc_file = "rcv1.{}.txt".format(self.kind)
    
    def download(self, directory="", shuffle=False, test_size=0, sample_count=0, keep_raw=False, force=False):
        if test_size != 0:
            raise Exception("This dataset is already splitted to train & test.")
        
        return super().download(directory, False, 0, sample_count, keep_raw, force)

    @classmethod
    def topics(cls):
        return ReutersNews("topics")

    @classmethod
    def industries(cls):
        return ReutersNews("industries")

    @classmethod
    def regions(cls):
        return ReutersNews("regions")

    def read_label_desc(self, path):
        with open(path, "r", encoding="utf-8") as f:
            for i, line in enumerate(f):
                if i == 0:
                    continue  # skip header
                ln = line.strip().split("\t")
                


    def extract(self, path):
        dir, fname = os.path.split(path)
        extracteds = self.extract_file(
            path, 
            [
                "RCV1-v2/lyrl2004_tokens_train.csv", 
                "RCV1-v2/lyrl2004_tokens_test.csv", 
                "RCV1-v2/" + self.label_desc_file
                ],
            remove=True
        )
        
        label_file_path = os.path.join(dir, self.label_file)
        if not os.path.exists(label_file_path):
            self.logger.info("Downloading the annotation file")
            dl_file_path = label_file_path + ".gz"
            self._download_file(self.label_url, dl_file_path)
            label_file_path = self.extract_file(dl_file_path, [self.label_file], remove=True)[0]

        self.logger.info("Reading the annotation file")
        annotations = {}
        annotation_count = self.get_line_count(label_file_path)
        with open(label_file_path, "r", encoding="utf-8") as f:
            for line in xtqdm(f, total=annotation_count):
                a = line.strip().split(" ")
                cat = a[0]
                document_id = a[1]
                if document_id in annotations:
                    annotations[document_id] += [cat]
                else:
                    annotations[document_id] = [cat]
        descs = ReutersNewsResource.read_descriptions(dir, self.kind)

        self.logger.info("Make annotated file")
        pathes = []
        for t in ["train", "test"]:
            file_path = os.path.join(dir, "{}_{}.txt".format(self.kind, t))
            self.logger.info("Annotating the {} file".format(t))
            data_path = os.path.join(dir, "lyrl2004_tokens_{}.csv".format(t))
            total_count = self.get_line_count(data_path)
            
            with open(file_path, "w", encoding="utf-8") as f:
                with open(data_path, "r", encoding="utf-8") as df:
                    for line in xtqdm(df, total=total_count):
                        doc_id, words = line.strip().split(",")
                        if doc_id in annotations:
                            ann = " ".join(annotations[doc_id])
                            if self.kind == "regions":
                                f.write("\t".join([ann, words]) + "\n")
                            else:
                                ps = [descs[a].parent for a in annotations[doc_id]]
                                ps = [p for p in ps if p not in ["Root", "None"]]
                                ps = list(set(ps))
                                ps = " ".join(ps)
                                f.write("\t".join([ann, ps, words]) + "\n")
            pathes.append(file_path)
            os.remove(data_path)
        os.remove(label_file_path)

        return pathes[0]

    def _download_file(self, url, file_path):
        r = requests.get(url)

        total_size = int(r.headers.get("content-length", 0))
        with open(file_path, "wb") as f:
            chunk_size = 1024
            limit = total_size / chunk_size
            for data in xtqdm(r.iter_content(chunk_size=chunk_size), total=limit, unit="B", unit_scale=True):
                f.write(data)

        return file_path

    def make_resource(self, data_root):
       return ReutersNewsResource(data_root, self.kind)

  
class ReutersNewsResource(Resource):

    def __init__(self, root, kind):
        columns = []
        target = ""
        if kind == "topics":
            columns = ["topic", "parent-topic", "words"]
            target = "topic"
        elif kind == "industries":
            columns = ["category", "2nd-level-topic", "words"]
            target = "category"
        elif kind == "regions":
            columns = ["region", "words"]
            target = "region"            

        super().__init__(
            root, 
            columns,
            target)
        
        self._resource = {
            "data": os.path.join(self.root, "{}_train.txt".format(kind)),
            "train": os.path.join(self.root, "{}_train.txt".format(kind)),
            "test": os.path.join(self.root, "{}_test.txt".format(kind)),
            "sample": os.path.join(self.root, "{}_train_samples.txt".format(kind))
        }
        self.descs = self.read_descriptions(self.root, kind)

    @classmethod
    def read_descriptions(cls, root, kind):
        Description = namedtuple("Description", ["code", "desc", "parent"])
        path = os.path.join(root, "rcv1.{}.txt".format(kind))
        descs = {}
        with open(path, "r", encoding="utf-8") as f:
            for i, line in enumerate(f):
                if i == 0:
                    continue
                els = line.strip().split("\t")
                key = els[0]
                parent = "" if kind == "regions" else els[-2]
                desc = "" if kind == "regions" else els[-1]
                d = Description(key, desc, parent)
                descs[key] = d
        
        return descs



import os
import zipfile
import shutil
from tqdm import tqdm
from chazutsu.datasets.framework.dataset import Dataset
from chazutsu.datasets.framework.resource import Resource


class CustomerReview(Dataset):

    def __init__(self, kind="products5"):
        super().__init__(
            name="Customer Review Data",
            site_url="https://www.cs.uic.edu/~liub/FBS/sentiment-analysis.html#datasets",
            download_url="",
            description="customer review data from amazon.com that is annotated to each sentences"
            )
        
        urls = {
            "products5": "http://www.cs.uic.edu/~liub/FBS/CustomerReviewData.zip",
            "additional9": "https://s3-ap-northeast-1.amazonaws.com/dev.tech-sketch.jp/chakki/chazutsu/Reviews-9-products.zip",
            "more3": "https://s3-ap-northeast-1.amazonaws.com/dev.tech-sketch.jp/chakki/chazutsu/CustomerReviews-3-domains.zip",
        }

        if kind not in urls:
            raise Exception("You have to choose kind from {}".format(",".join(urls.keys())))

        self.kind = kind
        self.download_url = urls[self.kind]
    
    @classmethod
    def products5(cls):
        return CustomerReview("products5")

    @classmethod
    def additional9(cls):
        return CustomerReview("additional9")

    @classmethod
    def more3(cls):
        return CustomerReview("more3")

    def extract(self, path):
        if self.kind == "products5":
            return self._extract_products5(path)
        elif self.kind == "additional9":
            return self._extract_additional9(path)
        elif self.kind == "more3":
            return self._extract_more3(path)
        else:
            raise Exception("Directed kind {} is not supported in extraction process.".format(self.kind))

    def make_resource(self, data_root):
        return Resource(data_root, columns=["sentence-type", "polarity", "detail", "review"], target="polarity")

    def _extract_products5(self, path):
        dir, file_name = os.path.split(path)
        work_dir = os.path.join(dir, "tmp")
        products5_path = os.path.join(dir, "products5.txt")

        with zipfile.ZipFile(path) as z:
            z.extractall(path=work_dir)

        review_path = os.path.join(work_dir, "customer review data")
        with open(products5_path, mode="wb") as f:
            for txt in os.listdir(review_path):
                if txt == "Readme.txt":
                    continue

                self.logger.info("Extracting {} data.".format(txt))
                skipped = []
                p = os.path.join(review_path, txt)
                total = self.get_line_count(p)
                with open(p, encoding="utf-8") as rv:
                    for ln in tqdm(rv, total=total):
                        _ln = ln.strip().replace("\t", " ")
                        r = ReviewSentence.parse(_ln)
                        if r is not None:
                            if r.sentence_type:
                                f.write((r.to_row() + "\n").encode("utf-8"))
                            else:
                                skipped.append(_ln)
        
                if len(skipped) > 0:
                    self.logger.warning(
                        " {} lines is skipped because of annotation format is not correct.".format(len(skipped))
                        )
                    self.logger.debug(
                        "\n".join(map(lambda s: " >>{}".format(s), skipped))
                        )

        # remove files
        os.remove(path)
        shutil.rmtree(work_dir)

        return products5_path

    def _extract_additional9(self, path):
        dir, file_name = os.path.split(path)
        work_dir = os.path.join(dir, "tmp")
        additional9_path = os.path.join(dir, "additional9.txt")

        with zipfile.ZipFile(path) as z:
            z.extractall(path=work_dir)

        review_path = os.path.join(work_dir, "Reviews-9-products")
        with open(additional9_path, mode="wb") as f:
            for txt in os.listdir(review_path):
                if txt == "Readme.txt":
                    continue

                self.logger.info("Extracting {} data.".format(txt))
                skipped = []
                p = os.path.join(review_path, txt)
                total = self.get_line_count(p)
                with open(p, encoding="utf-8", errors="replace") as rv:
                    for ln in tqdm(rv, total=total):
                        _ln = ln.strip().replace("\t", " ")
                        r = ReviewSentence.parse(_ln)
                        if r is not None:
                            if r.sentence_type:
                                f.write((r.to_row() + "\n").encode("utf-8"))
                            else:
                                skipped.append(_ln)
        
                if len(skipped) > 0:
                    self.logger.warning(
                        " {} lines is skipped because of annotation format is not correct.".format(len(skipped))
                        )
                    self.logger.debug(
                        "\n".join(map(lambda s: " >>{}".format(s), skipped))
                        )

        # remove files
        os.remove(path)
        shutil.rmtree(work_dir)

        return additional9_path

    def _extract_more3(self, path):
        dir, file_name = os.path.split(path)
        work_dir = os.path.join(dir, "tmp")
        more3_path = os.path.join(dir, "more3.txt")

        with zipfile.ZipFile(path) as z:
            z.extractall(path=work_dir)

        review_path = os.path.join(work_dir, "CustomerReviews-3domains(IJCAI2015)")
        with open(more3_path, mode="wb") as f:
            for txt in os.listdir(review_path):
                if txt == "Readme.txt":
                    continue

                if txt.endswith(".xml"):
                    continue

                self.logger.info("Extracting {} data.".format(txt))
                skipped = []
                p = os.path.join(review_path, txt)
                total = self.get_line_count(p)
                with open(p, encoding="utf-8") as rv:
                    for ln in tqdm(rv, total=total):
                        _ln = ln.strip().replace("\t", " ")
                        r = ReviewSentence.parse(_ln)
                        if r is not None:
                            if r.sentence_type:
                                f.write((r.to_row() + "\n").encode("utf-8"))
                            else:
                                skipped.append(_ln)
        
                if len(skipped) > 0:
                    self.logger.warning(
                        " {} lines is skipped because of annotation format is not correct.".format(len(skipped))
                        )
                    self.logger.debug(
                        "\n".join(map(lambda s: " >>{}".format(s), skipped))
                        )

        # remove files
        os.remove(path)
        shutil.rmtree(work_dir)

        return more3_path


class ReviewSentence():

    def __init__(self, sentence_type, body, detail="", polarity=0):
        self.sentence_type = sentence_type
        self.detail = detail
        self.body = body
        self.polarity = polarity
    
    @classmethod
    def parse(cls, sentence):
        s = sentence.strip()
        if not s:
            return None

        if s.startswith("*"):
            return None  # comment
        
        if s.startswith("[t]"):
            return ReviewSentence("t", s[len("[t]"):])

        attr_body = s.split("##")
        if len(attr_body) != 2:
            attr_body = s.split("#")
            if len(attr_body) != 2:
                return ReviewSentence("", "")

        attr, body = attr_body
        if not attr:
            return ReviewSentence("-", body)
        
        attrs = attr.split(",")
        details = []
        scores = []
        for a in attrs:
            _a = a.strip()
            if not _a:
                continue
            detail = _a.replace("[", "_").replace("{", "_").replace("]", "").replace("}", "")
            ds = detail.split("_")

            if len(ds) == 2:  # no attribute
                detail += "_"
            elif len(ds) == 1: # annotation miss
                break
            
            if len(ds) > 1:
                if ds[1] == "+":
                    scores.append(1)
                elif ds[1] == "-":
                    scores.append(-1)
                elif ds[1].replace("+", "").replace("-", "").isdigit():
                    scores.append(int(ds[1]))
                else:
                    continue
            else:
                continue
            details.append(detail)
        
        if len(scores) == 0:
            return ReviewSentence("", "")

        score = sum(scores) / len(scores)
        return ReviewSentence("po", body, ",".join(details), score)
    
    def to_row(self):
        return "\t".join([
            self.sentence_type, 
            str(self.polarity),
            self.detail,
            self.body])
    


import os
import tarfile
import shutil
from collections import Counter
from chazutsu.datasets.framework.dataset import Dataset
from chazutsu.datasets.framework.resource import Resource


class WikiText103(Dataset):

    def __init__(self):
        super().__init__(
            name="WikiText-103",
            site_url="https://metamind.io/research/the-wikitext-long-term-dependency-language-modeling-dataset/",
            download_url="https://s3.amazonaws.com/research.metamind.io/wikitext/wikitext-103-v1.zip",
            description="The dataset for language modeling that is larger than PTB (over 110 times)."
            )
    
    def download(self, directory="", shuffle=False, test_size=0, sample_count=0, keep_raw=False, force=False):
        if test_size != 0:
            raise Exception("This dataset is already splitted to train & test.")
        
        return super().download(directory, False, 0, sample_count, keep_raw, force)

    def extract(self, path):
        dir, file_name = os.path.split(path)
        extracteds = self.extract_file(
            path, 
            [
                "wikitext-103/wiki.train.tokens", 
                "wikitext-103/wiki.test.tokens", 
                "wikitext-103/wiki.valid.tokens"
                ],
            remove=True
        )

        train_file_path = os.path.join(dir, "wiki.train.tokens")
        return train_file_path
    
    def make_resource(self, data_root):
        return Resource(
            data_root,
            ["sentence"],
            pattern={
                "train": ".train",
                "test": ".test",
                "valid": ".valid",
                "samples": "_samples"
            }
        )



import os
import shutil
from collections import Counter
from chazutsu.datasets.framework.dataset import Dataset
from chazutsu.datasets.framework.resource import Resource


class Text8(Dataset):

    def __init__(self):
        super().__init__(
            name="Text8",
            site_url="http://mattmahoney.net/dc/textdata",
            download_url="http://mattmahoney.net/dc/text8.zip",
            description="This dataset offers cleaned English Wikipedia text"
            )
        self._test_size = 0
    
    def download(self, directory="", shuffle=False, test_size=10, sample_count=0, keep_raw=False, force=False):
        if test_size < 1:
            raise Exception("This dataset is splitted by byte. test_size means how many mega bytes are used for test.")
        
        self._test_size = test_size
        return super().download(directory, False, 0, sample_count, keep_raw, force)

    def extract(self, path):
        dir, file_name = os.path.split(path)
        extracteds = self.extract_file(path, ["text8"], remove=True)

        train_file_path = os.path.join(dir, "text8.train.txt")
        if self._test_size == 0:
            os.rename(os.path.join(dir, "text8"), train_file_path)
        else:
            test_file_path = os.path.join(dir, "text8.test.txt")

            line = ""
            test_byte = self._test_size * 1000000
            with open(os.path.join(dir, "text8"), encoding="utf-8") as f:
                line = f.readline().strip()
            with open(train_file_path, mode="w", encoding="utf-8") as train:
                train.write(line[:-test_byte])
            with open(test_file_path, mode="w", encoding="utf-8") as test:
                test.write(line[-test_byte:])
        
            os.remove(os.path.join(dir, "text8"))

        return train_file_path
    
    def make_resource(self, data_root):
        return Resource(
            data_root,
            ["sentence"],
            pattern={
                "train": ".train",
                "test": ".test",
                "valid": ".valid",
                "samples": "_samples"
            }
        )



import os
import sys
import re
import mmap
import random
import math
import zipfile
import tarfile
import gzip
from urllib.parse import urlparse
import requests
from chazutsu.datasets.framework.xtqdm import xtqdm
from joblib import Parallel, delayed
from chazutsu.datasets.framework.resource import Resource


class Dataset():
    """ Dataset Class Framework """

    def __init__(self, name, site_url, download_url, description, log_level=None):
        """
        Args:
            name        : dataset name
            site_url    : url to access the dataset homepage
            download_url    : dataset download url
            description     : description of dataset
        """

        self.name = name
        self.site_url = site_url
        self.download_url = download_url
        self.description = description

        # create logger
        from logging import getLogger, StreamHandler, DEBUG
        self.logger = getLogger("_".join(["dataset", self.__class__.__name__.lower()]))
        if not self.logger.hasHandlers():
            # logger is global object!
            _level = DEBUG if log_level is None else log_level
            handler = StreamHandler(sys.stdout)
            handler.setLevel(_level)
            self.logger.setLevel(_level)
            self.logger.addHandler(handler)
    
    def download(self, directory="", shuffle=True, test_size=0.3, sample_count=0, keep_raw=False, force=False):
        # input parameter check
        #   directory: don't make default directory in download method because can not specify current dir.
        #   test_size: have to be smaller than 1
        dir = self.check_directory(directory)
        dataset_root = os.path.join(dir, self._get_root_name())
        if not os.path.isdir(dataset_root):
            os.mkdir(dataset_root)
        elif not force:
            # data_root already exists and have contents
            self.logger.info("Read resource from the existed resource (if you want to retry, set force=True).")
            return self.make_resource(dataset_root)

        # download and save file
        save_file_path = self.save_dataset(dataset_root)

        # extract dataset file from saved file
        extracted_file_path = self.extract(save_file_path)

        if shuffle:
            self.logger.info("Shuffle the extracted dataset.")
            lines = []
            with open(extracted_file_path, encoding="utf-8") as f:
                lines = f.readlines()
            random.shuffle(lines)
            with open(extracted_file_path, "w", encoding="utf-8") as f:
                f.writelines(lines)

        # make sample file
        sample_path = self.make_samples(extracted_file_path, sample_count)

        # split to train & test
        train_test_path = self.train_test_split(extracted_file_path, test_size, keep_raw)
        
        self.logger.info("Done all process! Make below files at {}".format(dataset_root))
        for f in os.listdir(dataset_root):
            self.logger.info(" " + f)
        
        r = self.make_resource(dataset_root)

        return r
    
    def load(self, directory=""):
        dir = self.check_directory(directory)
        dataset_root = os.path.join(dir, self._get_root_name())
        if os.path.isdir(dataset_root):
            return self.make_resource(dataset_root)
        else:
            return None
    
    def _get_root_name(self):
        return self.name.lower().replace(" ", "_")

    def check_directory(self, directory):
        if os.path.isdir(directory):
            return directory
        else:
            current = os.getcwd()
            self.logger.info("Make directory for downloading the file to {}.".format(current))
            data_dir = os.path.join(current, "data")
            if not os.path.isdir(data_dir):
                os.mkdir(data_dir)
            return data_dir

    def save_dataset(self, dataset_root):
        save_file_path = os.path.join(dataset_root, self._get_file_name(None))
        if os.path.exists(save_file_path):
            self.logger.info("Skip the downloading the file because it already exists.")
            return save_file_path

        # download and save it as raw file
        self.logger.info("Begin downloading the {} dataset from {}.".format(self.name, self.download_url))
        resp = requests.get(self.download_url, stream=True)
        if not resp.ok:
            raise Exception("Can not get dataset from {}.".format(self.download_url))
        
        # save content in response to file
        total_size = int(resp.headers.get("content-length", 0))
        file_name = self._get_file_name(resp)
        _, ext = os.path.splitext(file_name)
        save_file_path = os.path.abspath(os.path.join(dataset_root, file_name))
        self.logger.info("The dataset file is saved to {}".format(save_file_path))
        with open(save_file_path, "wb") as f:
            chunk_size = 1024
            limit = total_size / chunk_size
            for data in xtqdm(resp.iter_content(chunk_size=chunk_size), total=limit, unit="B", unit_scale=True):
                f.write(data)
        
        return save_file_path
    
    def extract(self, path):
        # you may unpack the file and extract data file.
        return path
    
    def extract_file(self, path, relative_pathes, remove=True):
        # unpack the archive file and extract directed path file
        base, _ = os.path.splitext(path)
        target = relative_pathes if isinstance(relative_pathes, (tuple, list)) else [relative_pathes]

        extracteds = []
        if zipfile.is_zipfile(path):
            with zipfile.ZipFile(path) as z:
                for n in filter(lambda n: n in target, z.namelist()):
                    file_name = os.path.basename(n)
                    p = os.path.join(os.path.dirname(path), file_name)
                    with open(p, "wb") as f:
                        f.write(z.read(n))
                    extracteds.append(p)

        elif tarfile.is_tarfile(path):
            with tarfile.open(path) as t:
                for m in filter(lambda m: m in target, t.getnames()):
                    file_name = os.path.basename(m)
                    p = os.path.join(os.path.dirname(path), file_name)
                    with open(p, "wb") as f:
                        with t.extractfile(m) as tf:
                            for c in tf:
                                f.write(c)
                    extracteds.append(p)
        
        elif path.endswith(".gz"):
            with gzip.open(path, "rb") as g:
                p = os.path.join(os.path.dirname(path), relative_pathes[0])
                extracteds.append(p)
                with open(p, "wb") as f:
                    for ln in g:
                        f.write(ln)

        if remove:
            # remove downloaded raw file (zip/tar.gz etc)
            os.remove(path)

        return extracteds
    
    def label_by_dir(self, file_path, target_dir, dir_and_label, task_size=10):

        label_dirs = dir_and_label.keys()
        dirs = [d for d in os.listdir(target_dir) if os.path.isdir(os.path.join(target_dir, d)) and d in label_dirs]

        write_flg = True
        for d in dirs:
            self.logger.info("Extracting {} directory files (labeled by {}).".format(d, dir_and_label[d]))
            label = dir_and_label[d]
            dir_path = os.path.join(target_dir, d)
            pathes = [os.path.join(dir_path, f) for f in os.listdir(dir_path)]
            pathes = [p for p in pathes if os.path.isfile(p)]
            task_length = int(math.ceil(len(pathes) / task_size))
            for i in xtqdm(range(task_length)):
                index = i * task_size
                task_packet = pathes[index:(index + task_size)]
                lines = Parallel(n_jobs=-1)(delayed(self._parallel_parser)(label, t) for t in task_packet)
                with open(file_path, mode="w" if write_flg else "a", encoding="utf-8") as f:
                    for ln in lines:
                        f.write(ln)
                write_flg = False
    
    @classmethod
    def _parallel_parser(cls, label, path):
        features = cls._file_to_features(path)
        line = "\t".join([str(label)] + features) + "\n"
        return line

    @classmethod
    def _file_to_features(cls, path):
        # override this method if you want implements custome process
        fs = []
        with open(path, encoding="utf-8") as f:
            lines = f.readlines()
            lines = [ln.replace("\t", " ").strip() for ln in lines]
            fs = [" ".join(lines)]
        return fs

    def train_test_split(self, original_file_path, test_size, keep_raw=False):
        if test_size < 0 or test_size > 1:
            self.logger.error("test_size have to be between 0 ~ 1. if you don't want to split, please set 0.")
            return []
        elif test_size == 0 or test_size == 1:
            return []

        self.logger.info("Split to train & test file.")

        total_count = self.get_line_count(original_file_path)
        test_count = int(round(total_count * test_size))
        test_targets = random.sample(range(total_count), test_count)
        
        base, ext = os.path.splitext(original_file_path)
        train_test_path = [base + x + ext for x in ["_train", "_test"]]
        train_file = open(train_test_path[0], "wb")
        test_file = open(train_test_path[1], "wb")

        with open(original_file_path, "rb") as f:
            i = 0
            for line in xtqdm(f, total=total_count):
                target = test_file if i in test_targets else train_file
                target.write(line)
                i += 1

        train_file.close()
        test_file.close()

        to_base = lambda x: os.path.basename(x)
        self.logger.info("File is splited to {} & {}. Each records are {} & {} (test_size={:.2f}%).".format(
            to_base(train_test_path[0]), to_base(train_test_path[1]),
            total_count - test_count, test_count, test_count / total_count * 100)
            )

        if not keep_raw:
            os.remove(original_file_path)

        return train_test_path

    def make_samples(self, original_file_path, sample_count):
        if sample_count == 0:
            return ""
        
        base, ext = os.path.splitext(original_file_path)
        sample_path = base + "_samples" + ext
        samples_file = open(sample_path, "wb")

        total_count = self.get_line_count(original_file_path)
        # for reproducibility of sampling, use fixed interval
        sample_target = range(0, total_count, total_count // sample_count)
        with open(original_file_path, "rb") as f:
            count = 0
            i = 0
            for line in f:
                if i in sample_target:
                    samples_file.write(line)
                    count += 1
                i += 1
                if count == sample_count:
                    break

        samples_file.close()
        self.logger.info("Make {} by picking {} records from original file.".format(
            os.path.basename(sample_path), sample_count)
        )

        return sample_path
    
    def make_resource(self, data_root):
        return Resource(data_root)

    def _get_file_name(self, resp):
        file_name = ""
        if resp and "content-disposition" in resp.headers:
            cd = resp.headers["content-disposition"]
            file_matches = re.search("filename=(.+)", cd)
            if file_matches:
                file_name = file_matches.group(0)
                file_name = file_name.split("=")[1]
        else:
            parsed = urlparse(self.download_url)
            file_name = os.path.basename(parsed.path)

        return file_name

    def get_line_count(self, file_path):  
        count = 0
        with open(file_path, "r+") as fp:
            buf = mmap.mmap(fp.fileno(), 0)
            while buf.readline():
                count += 1
        return count

    def show(self):
        print("About {}".format(self.name))
        print(self.description)
        print("see also: {}".format(self.site_url))




def get_xtqdm():
    from tqdm import tqdm
    from tqdm import tqdm_notebook

    if is_jupyter():
        return tqdm_notebook
    else:
        return tqdm

def is_jupyter():
    try:
        shell = get_ipython().__class__.__name__
        if shell == "ZMQInteractiveShell":
            return True
        elif shell == "TerminalInteractiveShell":
            return False
        else:
            return False
    except NameError:
        return False

xtqdm = get_xtqdm()



import os
import tarfile
import shutil
from collections import Counter
from chazutsu.datasets.framework.dataset import Dataset
from chazutsu.datasets.framework.resource import Resource


class PTB(Dataset):

    def __init__(self):
        super().__init__(
            name="Penn Tree Bank",
            site_url="https://github.com/tomsercu/lstm",
            download_url="https://s3-ap-northeast-1.amazonaws.com/dev.tech-sketch.jp/chakki/chazutsu/ptb.zip",
            description="basic language modeling dataset that omits linguistic structure annotations."
            )
    
    def download(self, directory="", shuffle=False, test_size=0, sample_count=0, keep_raw=False, force=False):
        if test_size != 0:
            raise Exception("This dataset is already splitted to train & test.")
        # in language modeling, shuffle is not needed
        return super().download(directory, False, 0, sample_count, keep_raw, force)

    def extract(self, path):
        dir, file_name = os.path.split(path)
        extracteds = self.extract_file(
            path, 
            ["ptb/ptb.train.txt", "ptb/ptb.valid.txt", "ptb/ptb.test.txt"],
            remove=True
        )
        train_file_path = os.path.join(dir, "ptb.train.txt")

        return train_file_path
    
    def make_resource(self, data_root):
        return Resource(
            data_root,
            ["sentence"],
            pattern={
                "train": ".train",
                "test": ".test",
                "valid": ".valid",
                "samples": "_samples"
            }
        )



import os
import re
import json
from collections import Counter
from chazutsu.datasets.framework.xtqdm import xtqdm
from chazutsu.datasets.framework.dataset import Dataset
from chazutsu.datasets.framework.resource import Resource, IndexedResource


class MultiNLI(Dataset):
    LABEL_MAP = {
        "entailment": 0,
        "neutral": 1,
        "contradiction": 2,
        "hidden": 0
    }

    def __init__(self, matched=True, full=False):
        """
        matched: matched dataset for mismatched dataset
        full: use full columns in the dataset. If full=False then extract only "gold_label", "genre", "sentence1", "sentence2"
        """
        super().__init__(
            name="MultiNLI",
            site_url="http://www.nyu.edu/projects/bowman/multinli/",
            download_url="https://s3-ap-northeast-1.amazonaws.com/dev.tech-sketch.jp/chakki/chazutsu/multinli_0.9_jsonl.zip",
            description="multi genre's text entailment dataset. There are in-domain(matched) and cross-domain(mismatched) data."
            )
        self.matched = matched
        self.is_full = full
        self.columns = []
        if self.is_full:
            self.columns = ["label", "annotator_labels", "genre", "pairID", "sentence1", "sentence2", "sentence1_parse", "sentence2_parse"]
        else:
            self.columns = ["label", "genre", "pairID", "sentence1", "sentence2"]
        self._tokenize_pattern = re.compile("\(|\)")
    
    def download(self, directory="", shuffle=True, test_size=0, sample_count=0, keep_raw=False, force=False):
        if test_size != 0:
            raise Exception("This dataset is already splitted to train & test.")
        # in language modeling, shuffle is not needed
        return super().download(directory, shuffle, 0, sample_count, keep_raw, force)

    @classmethod
    def matched(cls, full=False):
        return MultiNLI(True, full)

    @classmethod
    def mismatched(cls, full=False):
        return MultiNLI(False, full)

    def extract(self, path):
        dir, file_name = os.path.split(path)
        kind = "matched" if self.matched else "mismatched"
        extracteds = self.extract_file(
            path, 
            [
                "multinli_0.9_jsonl/multinli_0.9_train.jsonl", 
                "multinli_0.9_jsonl/multinli_0.9_{}_dev.jsonl".format(kind),
                "multinli_0.9_jsonl/multinli_0.9_{}_unlabeled_test.jsonl".format(kind)
            ],
            remove=True
        )

        train_file = ""
        for e in extracteds:
            preprocessed = self.preprocess_file(e)
            os.remove(e)
            if "_train" in preprocessed:
                train_file = preprocessed

        return preprocessed
    
    def preprocess_file(self, path):
        write_file_path = path.replace(".jsonl", ".txt")
        write_file = open(write_file_path, mode="w", encoding="utf-8")
        file_kind = path.split("_")[-1]

        self.logger.info("Preprocessing {} file".format(file_kind))
        total_count = self.get_line_count(path)
        with open(path, encoding="utf-8") as rf:
            for line in xtqdm(rf, total=total_count):
                preprocessed = self.preprocess_jsonl(line)
                if preprocessed:
                    w_line = "\t".join(preprocessed) + "\n"
                    write_file.write(w_line)
        
        write_file.close()
        return write_file_path

    def preprocess_jsonl(self, jsonl):
        loaded = json.loads(jsonl)
        if loaded["gold_label"] not in self.LABEL_MAP:
            return None

        loaded["label"] = str(self.LABEL_MAP[loaded["gold_label"]])
        if "genre" not in loaded or not loaded["genre"]:
            loaded["genre"] = "-"
        loaded["sentence1"] = self._tokenized_str(loaded["sentence1_binary_parse"])
        loaded["sentence2"] = self._tokenized_str(loaded["sentence2_binary_parse"])

        values = [loaded[c] for c in self.columns]
        return values

    def _tokenized_str(self, sentence):
        _s = self._tokenize_pattern.sub("", sentence)
        tokens = _s.split()
        tokens = [t.strip() for t in tokens if t.strip()]
        return " ".join(tokens)

    def make_resource(self, data_root):
        return NLIResource(data_root, columns=self.columns, target="label")


class NLIResource(Resource):

    def __init__(self, 
        root,
        columns=None,
        target="",
        separator="\t",
        pattern=()):

        super().__init__(
            root, 
            columns,
            target,
            separator,
            {
                "train": "_train",
                "test": "_test",
                "dev": "_dev",
                "sample": "_samples"
            })
    
    @property
    def dev_file_path(self):
        return self._get_prop("dev")
    
    def dev_data(self, split_target=False):
        return self._get_data("dev", split_target)

    def to_indexed(self, vocab_resources=("train", "dev", "test"), vocab_columns=("sentence1", "sentence2")):
        ir = IndexedResource(self, vocab_resources, vocab_columns)
        return ir



import os
import tarfile
import shutil
from chazutsu.datasets.framework.xtqdm import xtqdm
from chazutsu.datasets.framework.dataset import Dataset
from chazutsu.datasets.framework.resource import Resource


class MovieReview(Dataset):

    def __init__(self, kind="polarity"):
        super().__init__(
            name="Moview Review Data",
            site_url="http://www.cs.cornell.edu/people/pabo/movie-review-data/",
            download_url="",
            description="movie review data that is annotated by 3 kinds of label (polarity, subjective rating, subjectivity)."
            )
        
        urls = {
            "polarity": "http://www.cs.cornell.edu/people/pabo/movie-review-data/review_polarity.tar.gz",
            "polarity_v1": "http://www.cs.cornell.edu/people/pabo/movie-review-data/rt-polaritydata.tar.gz",
            "rating": "http://www.cs.cornell.edu/people/pabo/movie-review-data/scale_data.tar.gz",
            "subjectivity": "http://www.cs.cornell.edu/people/pabo/movie-review-data/rotten_imdb.tar.gz"
        }

        if kind not in urls:
            raise Exception("You have to choose kind from {}".format(",".join(urls.keys())))

        self.kind = kind
        self.download_url = urls[self.kind]
    
    @classmethod
    def polarity(cls):
        return MovieReview("polarity")

    @classmethod
    def polarity_v1(cls):
        return MovieReview("polarity_v1")

    @classmethod
    def rating(cls):
        return MovieReview("rating")

    @classmethod
    def subjectivity(cls):
        return MovieReview("subjectivity")

    def extract(self, path):
        if self.kind == "polarity":
            return self._extract_polarity(path)
        elif self.kind == "polarity_v1":
            return self._extract_polarity_v1(path)
        elif self.kind == "rating":
            return self._extract_rating(path)
        elif self.kind == "subjectivity":
            return self._extract_subjectivity(path)
        else:
            raise Exception("Directed kind {} is not supported in extraction process.".format(self.kind))

    def make_resource(self, data_root):
        if self.kind in ["polarity", "polarity_v1"]:
            return Resource(data_root, columns=["polarity", "review"], target="polarity")
        elif self.kind == "rating":
            return Resource(data_root, columns=["rating", "review"], target="rating")
        elif self.kind == "subjectivity":
            return Resource(data_root, columns=["subjectivity", "review"], target="subjectivity")
        else:
            return Resource(data_root)

    def _extract_polarity(self, path):
        dir, file_name = os.path.split(path)
        work_dir = os.path.join(dir, "tmp")
        polarity_file_path = os.path.join(dir, "review_polarity.txt")

        with tarfile.open(path) as t:
            t.extractall(path=work_dir)

        negative_path = os.path.join(work_dir, "txt_sentoken/neg")
        positive_path = os.path.join(work_dir, "txt_sentoken/pos")

        with open(polarity_file_path, mode="w", encoding="utf-8") as f:
            for i, p in enumerate([negative_path, positive_path]):
                label = i # negative = 0, positive = 1
                self.logger.info(
                    "Extracting {} data.".format("negative" if label == 0 else "positive")
                    )
                for txt in xtqdm(os.listdir(p)):
                    with open(os.path.join(p, txt), encoding="utf-8") as tf:
                        lines = [ln.strip().replace("\t", " ") for ln in tf.readlines()]
                        review= " ".join(lines)
                        f.write("\t".join([str(label), review]) + "\n")

        # remove files
        os.remove(path)
        shutil.rmtree(work_dir)

        return polarity_file_path

    def _extract_polarity_v1(self, path):
        dir, file_name = os.path.split(path)
        extracteds = self.extract_file(
            path, 
            ["rt-polaritydata/rt-polarity.neg","rt-polaritydata/rt-polarity.pos"],
            remove=True
        )

        polarity_file = os.path.join(dir, "review_polarity_v1.txt")
        with open(polarity_file, mode="w", encoding="utf-8") as f:
            for e in extracteds:
                label = 0 if e.endswith(".neg") else 1
                self.logger.info(
                    "Extracting {} data.".format("negative" if label == 0 else "positive")
                    )
                total = self.get_line_count(e)
                with open(e, mode="r", errors="replace", encoding="utf-8") as p:
                    for ln in xtqdm(p, total=total):
                        review = ln.strip().replace("\t", " ")
                        f.write("\t".join([str(label), review]) + "\n")
        
        for e in extracteds:
            os.remove(e)
        
        return polarity_file

    def _extract_rating(self, path):
        dir, file_name = os.path.split(path)
        work_dir = os.path.join(dir, "tmp")
        rating_file_path = os.path.join(dir, "review_rating.txt")

        with tarfile.open(path) as t:
            t.extractall(path=work_dir)
        
        rating_dir = os.path.join(work_dir, "scaledata")

        rating_file = open(rating_file_path, "w", encoding="utf-8")
        for user in os.listdir(rating_dir):
            user_dir = os.path.join(rating_dir, user)
            if not os.path.isdir(user_dir):
                continue
            
            sub_in_review_file = os.path.join(user_dir, "subj." + user)
            user_rating_file = os.path.join(user_dir, "rating." + user)
            total = self.get_line_count(sub_in_review_file)
            self.logger.info("Extracting user {}'s rating data.".format(user))
            with open(sub_in_review_file, "r", encoding="utf-8") as sr:
                with open(user_rating_file, "r", encoding="utf-8") as ur:
                    for review, rating in xtqdm(zip(sr, ur), total=total):
                        _rv = review.strip().replace("\t", " ")
                        _r = rating.strip()
                        rating_file.write("\t".join([_r, _rv]) + "\n")
        
        rating_file.close()
        os.remove(path)
        shutil.rmtree(work_dir)

        return rating_file_path

    def _extract_subjectivity(self, path):
        dir, file_name = os.path.split(path)
        extracteds = self.extract_file(
            path, 
            ["plot.tok.gt9.5000","quote.tok.gt9.5000"],
            remove=True
        )

        subjectivity_file = os.path.join(dir, "subjectivity.txt")
        with open(subjectivity_file, mode="w", encoding="utf-8") as f:
            for e in extracteds:
                fname = os.path.basename(e)
                label = 1 if fname.startswith("plot.") else 0  # subjective(plot) = 1
                self.logger.info(
                    "Extracting {} data.".format("subjective" if label == 1 else "objective")
                    )
                total = self.get_line_count(e)
                with open(e, mode="r", errors="replace", encoding="utf-8") as sb:
                    for ln in xtqdm(sb, total=total):
                        review = ln.strip().replace("\t", " ")
                        f.write("\t".join([str(label), review]) + "\n")
        
        for e in extracteds:
            os.remove(e)
        
        return subjectivity_file



import os
import sys
import mmap
from collections import Counter
from chazutsu.datasets.framework.xtqdm import xtqdm
from chazutsu.datasets.framework.tokenizer import Tokenizer


class Vocabulary():

    def __init__(self, root, name, tokenizer=None, end_of_sentence="", unknown="<unk>"):
        self.root = root
        self.name = name
        self._vocab_file_path = os.path.join(root, name + ".vocab")
        self.tokenizer = tokenizer
        if self.tokenizer is None:
            self.tokenizer = Tokenizer()
        self.end_of_sentence = end_of_sentence
        self.unknown = unknown
        self._vocab = {}
        self.__rev_vocab = {}

        # create logger
        from logging import getLogger, StreamHandler, DEBUG
        self.logger = getLogger(self.name.lower() + "_vocabulary")
        if not self.logger.hasHandlers():
            # logger is global object!
            _level = DEBUG
            handler = StreamHandler(sys.stdout)
            handler.setLevel(_level)
            self.logger.setLevel(_level)
            self.logger.addHandler(handler)

        if self.has_vocab():
            self.load()

    def has_vocab(self):
        return os.path.exists(self._vocab_file_path)

    def make(self, path_or_paths, vocab_size=-1, min_word_count=0, target_column_indexes=(), separator="\t"):
        vocab = Counter()
        paths = path_or_paths
        if isinstance(paths, str):
            paths = [paths]

        for p in paths:
            self.logger.info("Read {} to make vocabulary.".format(p))
            count = self.get_line_count(p)
            for words in xtqdm(self.fetch_line(p, target_column_indexes, separator), total=count):
                for w in words:
                    vocab[w] += 1
        
        _vocab = [k_v[0] for k_v in vocab.most_common() if not k_v[1] < min_word_count]
        if vocab_size > 0:
            _vocab = _vocab[:vocab_size]

        if self.unknown and self.unknown not in _vocab:
            _vocab.append(self.unknown)
        if self.end_of_sentence and self.end_of_sentence not in _vocab:
            _vocab.append(self.end_of_sentence)

        self.logger.info("The vocabulary count is {}. You can see it in {}.".format(
            len(_vocab), self._vocab_file_path
            ))
        with open(self._vocab_file_path, "w", encoding="utf-8") as f:
            f.write("\n".join(_vocab))
        self._vocab = dict(zip(_vocab, range(len(_vocab))))
        self.__rev_vocab = {}

    def get_line_count(self, file_path):  
        count = 0
        with open(file_path, "r+") as fp:
            buf = mmap.mmap(fp.fileno(), 0)
            while buf.readline():
                count += 1
        return count

    def fetch_line(self, path, target_column_indexes=(), separator="\t"):
        with open(path, encoding="utf-8") as f:
            for line in f:
                _line = line
                if len(target_column_indexes) > 0:
                    els = _line.split(separator)
                    els = [els[i] for i in target_column_indexes]
                    _line = " ".join(els)

                words = self.tokenizer.tokenize(_line)
                if self.end_of_sentence:
                    words.append(self.end_of_sentence)

                yield words
    
    def load(self):
        if self.has_vocab():
            self.__rev_vocab = {}
            with open(self._vocab_file_path, encoding="utf-8") as f:
                lines = f.readlines()
                for i, ln in enumerate(lines):
                    w = ln.strip()
                    self._vocab[w] = i

    def str_to_ids(self, sentence):
        if len(self._vocab) == 0:
            self.load()
        words = self.tokenizer.tokenize(sentence)
        unk_id = self._vocab[self.unknown]
        ids = [unk_id if w not in self._vocab else self._vocab[w] for w in words]
        if self.end_of_sentence and sentence.endswith(os.linesep):
            eos_id = self._vocab[self.end_of_sentence]
            ids.append(eos_id)

        return ids

    def ids_to_words(self, ids):
        if len(self._vocab) == 0:
            self.load()
        
        if len(self.__rev_vocab) == 0:
            self.__rev_vocab = {v:k for k, v in self._vocab.items()}

        words = [self.__rev_vocab[i] for i in ids]
        return words



import os
import re


class Tokenizer():

    def __init__(self):
        self._line_strip_pattern = re.compile("(\.|\!|\?|\"|\'|,|\(|\)|\<|\>)")
    
    def tokenize(self, sentence):
        stripped = self.line_strip(sentence)
        words = self._tokenize(stripped)
        words = [w.strip() for w in words]
        words = [w for w in words if w]
        return words

    def line_strip(self, sentence):
        return re.sub(self._line_strip_pattern, " ", sentence)
    
    def _tokenize(self, stripped_sentence):
        # You have to implement how to tokenize the sentence
        words = stripped_sentence.split(" ") 
        return words
    


import os
import pandas as pd
from chazutsu.datasets.framework.vocabulary import Vocabulary


class Resource():

    def __init__(self, 
        root,
        columns=None,
        target="",
        separator="\t",
        pattern=()):

        self.root = root
        self.columns = columns
        self.target = target
        self.separator = separator
        self._pattern = pattern
        if len(self._pattern) == 0:
            self._pattern = {
                "train": "_train",
                "test": "_test",
                "valid": "_valid",
                "sample": "_samples"
            }
        self._resource_name = ""
        self._resource = {}

        if target and target not in columns:
            raise Exception("Target have to be selected from columns.")
        self.find_resource()
    
    def find_resource(self):
        for f in os.listdir(self.root):
            p = os.path.join(self.root, f)
            if not os.path.isfile(p) or f.startswith("."):
                continue
            
            n, e = os.path.splitext(f)
            if e == ".vocab":
                continue  # skip vocab file

            match = False
            for kind in self._pattern:
                if n.endswith(self._pattern[kind]):
                    self._resource[kind] = p
                    self._resource_name = n.replace(self._pattern[kind], "")
                    match = True
                    break

            if not match:
                self._resource["data"] = p
                self._resource_name = n
        
        if "data" not in self._resource:
            self._resource["data"] = self._resource["train"]

    @property
    def data_file_path(self):
        return self._get_prop("data")

    @property
    def train_file_path(self):
        return self._get_prop("train")

    @property
    def test_file_path(self):
        return self._get_prop("test")

    @property
    def valid_file_path(self):
        return self._get_prop("valid")

    @property
    def sample_file_path(self):
        return self._get_prop("sample")
    
    def _get_prop(self, name):
        return "" if name not in self._resource else self._resource[name]        

    def data(self, split_target=False):
        return self._get_data("data", split_target)
    
    def train_data(self, split_target=False):
        return self._get_data("train", split_target)

    def test_data(self, split_target=False):
        return self._get_data("test", split_target)

    def valid_data(self, split_target=False):
        return self._get_data("valid", split_target)

    def sample_data(self, split_target=False):
        return self._get_data("sample", split_target)

    def _get_data(self, name, split_target):
        if name not in self._resource:
            raise Exception("Can not find {} data resource.".format(name))
        return self._to_pandas(self._resource[name], split_target)

    def _to_pandas(self, path, split_target):
        df = pd.read_table(path, header=None, names=self.columns)

        if not split_target:
            return df
        elif self.target and self.target in df.columns:
            target = df[self.target]
            df.drop(self.target, axis=1, inplace=True)
            return target, df
        else:
            return df
    
    def to_indexed(self, vocab_resources=("train", "valid", "test"), vocab_columns=()):
        ir = IndexedResource(self, vocab_resources, vocab_columns)
        return ir


class IndexedResource(Resource):

    def __init__(self, 
        resource,
        vocab_resources=("train", "valid", "test"),
        vocab_columns=()):

        if len(resource._resource) == 0:
            raise Exception("supplied resource does not have any resource.")

        super().__init__(
            resource.root,
            resource.columns,
            resource.target,
            resource.separator,
            resource._pattern
        )
        self.vocab_resources = vocab_resources
        self.vocab_columns = vocab_columns
        if len(self.vocab_columns) == 0:
            self.vocab_columns = [self.columns[-1]]  # assume last column would be text

        self._resource_name = resource._resource_name
        self._original_resource = resource._resource
        self.vocab = Vocabulary(self.root, self._resource_name)

    def make_vocab(self, 
        tokenizer=None, 
        vocab_size=-1,
        min_word_count=0, 
        end_of_sentence="", 
        unknown="<unk>"):

        if tokenizer is not None:
            self.vocab.tokenizer = tokenizer
        if end_of_sentence:
            self.vocab.end_of_sentence = end_of_sentence
        if unknown:
            self.vocab.unknown = unknown
        
        paths = []
        for kind in self._original_resource:
            p = self._original_resource[kind]
            if p and kind in self.vocab_resources:
                paths.append(p)
        column_indexes = [i for i, c in enumerate(self.columns) if c in self.vocab_columns]

        if len(paths) > 0:
            self.vocab.make(paths, vocab_size, min_word_count, column_indexes, self.separator)
        
        return self

    @property
    def vocab_file_path(self):
        if self.vocab.has_vocab():
            return self.vocab._vocab_file_path
        else:
            return ""

    def vocab_data(self):
        v = self.vocab._vocab
        if len(v) == 0 and self.vocab.has_vocab():
            self.vocab.load()
            v = self.vocab._vocab        
        return v

    def str_to_ids(self, sentence):
        return self.vocab.str_to_ids(sentence)

    def ids_to_words(self, ids):
        return self.vocab.ids_to_words(ids)

    def _to_pandas(self, path, split_target):
        if not self.vocab.has_vocab():
            raise Exception("You have to make vocabulary by make_vocab first.")

        df = pd.read_table(path, header=None, names=self.columns)
        for c in self.vocab_columns:
            df[c] = df[c].map(self.vocab.str_to_ids)

        if not split_target:
            return df
        elif self.target and self.target in df.columns:
            target = df[self.target]
            df.drop(self.target, axis=1, inplace=True)
            return target, df
        else:
            return df
